using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
// for the Tablet SDK
using Microsoft.Ink;
// for ink serialization
using System.Text;

namespace StickyNoteSample
{
	/// <summary>
	/// Summary description for StickyNote.
	/// </summary>
	public class StickyNote : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.CheckBox checkBox1;

		// collects ink		
		private InkCollector myInkCollector;

		public StickyNote()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call

			// connect the ink collector to the form
			myInkCollector = new InkCollector(this.Handle);
			// set the width of the ink
			myInkCollector.DefaultDrawingAttributes.Width = 20;
			// turn on ink collection
			myInkCollector.Enabled = true;

		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.checkBox1 = new System.Windows.Forms.CheckBox();
			this.SuspendLayout();
			// 
			// checkBox1
			// 
			this.checkBox1.Location = new System.Drawing.Point(0, 8);
			this.checkBox1.Name = "checkBox1";
			this.checkBox1.Size = new System.Drawing.Size(64, 16);
			this.checkBox1.TabIndex = 0;
			this.checkBox1.Text = "erase";
			this.checkBox1.CheckedChanged += new System.EventHandler(this.checkBox1_CheckedChanged);
			// 
			// StickyNote
			// 
			this.BackColor = System.Drawing.Color.LemonChiffon;
			this.Controls.Add(this.checkBox1);
			this.Name = "StickyNote";
			this.Size = new System.Drawing.Size(200, 160);
			this.Load += new System.EventHandler(this.StickyNote_Load);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.StickyNote_MouseMove);
			this.ResumeLayout(false);

		}
		#endregion

		private void StickyNote_Load(object sender, System.EventArgs e)
		{
			
		}


		private void checkBox1_CheckedChanged(object sender, System.EventArgs e)
		{
			if (checkBox1.Checked)
				// set it to not collect ink
				myInkCollector.Enabled = false;
			else
				// set it to collect ink
				myInkCollector.Enabled = true;
		}

		private void StickyNote_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// if the collector is not collecting ink and the mouse button is pushed then erase
			if (!myInkCollector.Enabled && (MouseButtons.None != MouseButtons))
			{
				Point pt = new Point(e.X, e.Y);	

				// Convert the specified point from pixel to ink space coordinates
				myInkCollector.Renderer.PixelToInkSpace(this.Handle, ref pt);

				EraseStrokes(pt,null);
			}
		}
		/// <summary>
		/// Helper method that performs a hit test using the specified point.
		/// It deletes all strokes that were hit by the point
		/// </summary>
		/// <param name="pt">The point to use for hit testing</param>
		private void EraseStrokes(Point pt, Stroke currentStroke)
		{

			int HitTestRadius = 3;

			// remove ink strokes within the hit radius of the mouse
			Strokes strokesHit = myInkCollector.Ink.HitTest(pt, HitTestRadius);

			if (null!=currentStroke && strokesHit.Contains(currentStroke))
			{
				strokesHit.Remove(currentStroke);
			}

			// delete all strokes that are in the hit radius
			myInkCollector.Ink.DeleteStrokes(strokesHit);

			if (strokesHit.Count > 0)
			{
				// repaint the screen
				this.Refresh();
			}
		
		}

		/// <summary>
		/// Get/set the ink contents of the control
		/// </summary>
		public string Ink
		{
			get
			{					
				// This object will encode our byte data to a UTF8 string
				UTF8Encoding utf8 = new UTF8Encoding();

				byte[] base64ISF_bytes;
				string base64ISF_string;

				// Get the base64 encoded ISF
				base64ISF_bytes = myInkCollector.Ink.Save(PersistenceFormat.Base64InkSerializedFormat);

				// Convert it to a String
				base64ISF_string = utf8.GetString(base64ISF_bytes);
									
				// return the string without the null at the end (the null causes problems when inserting into the database)
				return base64ISF_string.Substring(0,base64ISF_string.Length-1);
			}
			set
			{
				// convert the base 64 string to a byte array and load it into the ink collector
				UTF8Encoding utf8 = new UTF8Encoding();		
				
				// place ink in a temporary ink container because you can't load into a collector that has already been inked
				Ink loadedInk = new Ink();
				loadedInk.Load(utf8.GetBytes(value));

				// clear the contents of the ink collector
				this.ClearInk();

				bool inkCollectorEnabled = myInkCollector.Enabled;
				// copy the temporary ink container into the ink collector
				myInkCollector.Enabled = false;
				myInkCollector.Ink = loadedInk;
				myInkCollector.Enabled = inkCollectorEnabled;
			}
		}
		/// <summary>
		/// Clear all ink
		/// </summary>
		public void ClearInk()
		{
			// clear out all strokes
			myInkCollector.Ink.DeleteStrokes();
			Refresh();

		}
	}
}
